<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Adapter;

use Exception;
use FiloBlu\MappConnector\Helper\FileLoggerHelper;
use FiloBlu\MappConnector\Model\Config;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\DataObject as DataObject;
use Magento\Framework\Filesystem;
use Magento\Framework\Filesystem\Io\Sftp;

/**
 * Class SFTPAdapter
 * @package FiloBlu\MappConnector\Adapter
 */
class SFTPAdapter implements MappAdapterInterface
{
    /**
     * Client
     *
     * @var
     */
    protected $_client;

    /**
     * Rest Config
     *
     * @var
     */
    protected $_config;

    /**
     * @var
     */
    protected $_systemconfig;

    /**
     * FileLogger Helper
     *
     * @var FileLoggerHelper
     */
    protected $_fileLogger;

    /**
     * @var array
     */
    private $_allowedExtensions = ['csv'];

    /**
     * RestAdapter constructor.
     */
    public function __construct()
    {
    }

    /**
     * Inits the Adapter
     *
     */
    public function _init()
    {
        $config = [
            "ftp_url" => $this->_systemconfig->getFTPUrl(),
            "ftp_login" => $this->_systemconfig->getFTPLogin(),
            "ftp_password" => $this->_systemconfig->getFTPPassword(),
        ];

        $this->setConfig($config);
    }

    /**
     * Sets the Adapter Config
     *
     * @param $config
     */
    public function setConfig($config)
    {
        $this->_config = $config;
    }

    /**
     * Gets the Adapter Config
     *
     * @return mixed
     */
    public function getConfig()
    {
        return $this->_config;
    }

    /**
     * Sets the System Config
     *
     * @param Config $config
     */
    public function setSystemConfig($config)
    {
        $this->_systemconfig = $config;
    }

    /**
     * Gets the System Config
     *
     * @return Config
     */
    public function getSystemConfig()
    {
        return $this->_systemconfig;
    }

    /**
     * Sets the File Logger
     *
     * @param $fileLogger
     */
    public function setFileLogger($fileLogger)
    {
        $this->_fileLogger = $fileLogger;
    }

    /**
     * Get Magento Var Path
     *
     * @return string
     */
    protected function _getMagentoVarPath()
    {
        $objectManager = ObjectManager::getInstance();
        $filesystem = $objectManager->create(Filesystem::class);
        $vardir = $filesystem
            ->getDirectoryRead(DirectoryList::VAR_DIR)
            ->getAbsolutePath();

        return $vardir . $this->_systemconfig->getLocalRootDir() . DIRECTORY_SEPARATOR . 'mapp-connector' . DIRECTORY_SEPARATOR;
    }

    /**
     * Makes the request
     *
     * @param DataObject $data
     * @return mixed
     * @throws Exception
     */
    public function makeRequest(DataObject $data)
    {
        $localDir = $this->_getMagentoVarPath() . $data['params']->config->normalized_type;
        $files = glob("$localDir/*");

        //If there is at least one file
        if (count($files)) {
            $this->_fileLogger->debug('There are some file to upload ', [], __METHOD__ . ':' . __LINE__);

            //Connect
            if ($this->_connect()) {

                //Move to remote dir
                if ($this->_moveToDir($data)) {

                    //Each file will be uploaded
                    foreach ($files as $file) {
                        $extension = pathinfo($file, PATHINFO_EXTENSION);
                        if (in_array($extension, $this->_allowedExtensions)) {
                            //Upload file in remote host
                            $this->_fileLogger->debug('Uploading File  ' . $file . ' ', [], __METHOD__ . ':' . __LINE__);
                            try {
                                if ($this->_client->write(basename($file), $file)) {
                                    $this->_fileLogger->debug('File ' . $file . ' uploaded and deleted from local path', [], __METHOD__ . ':' . __LINE__);
                                    //Delete file from local path
                                    unlink($file);
                                } else {
                                    $this->_fileLogger->error('There was an error uploading the file ' . $file, [], __METHOD__ . ':' . __LINE__);
                                    //Destroy SFTP Connection
                                    $this->__deconstruct();
                                    return false;
                                }
                            } catch (Exception $e) {
                                $this->_fileLogger->error($e->getMessage(), [], __METHOD__ . ':' . __LINE__);
                                //Destroy SFTP Connection
                                $this->__deconstruct();
                                return false;
                            }
                        } else {
                            $this->_fileLogger->error('Extension not allowed for file ' . $file, [], __METHOD__ . ':' . __LINE__);
                        }
                    }

                    //Destroy SFTP Connection
                    $this->__deconstruct();
                    return true;
                } else {
                    $this->_fileLogger->error('Error creating directory', [], __METHOD__ . ':' . __LINE__);
                }
            } else {
                $this->_fileLogger->error('Failed to connect to SFTP', [], __METHOD__ . ':' . __LINE__);
            }
        } else {
            $this->_fileLogger->debug('There are no file to upload for ' . $data['params']->config->normalized_type, [], __METHOD__ . ':' . __LINE__);
        }

        return false;
    }

    /**
     * SFTP Connect
     *
     * @return bool
     */
    public function _connect()
    {
        try {
            $this->_fileLogger->debug('Connecting to ' . $this->_config['ftp_url'] . ', for user ' . $this->_config['ftp_login'], [], __METHOD__ . ':' . __LINE__);
            $objectManager = ObjectManager::getInstance();
            $this->_client = $objectManager->create(Sftp::class);

            $this->_client->open(
                [
                    'host' => $this->_config['ftp_url'],
                    'username' => $this->_config['ftp_login'],
                    'password' => $this->_config['ftp_password'],
                ]
            );

            $this->_fileLogger->debug('Connected to ' . $this->_config['ftp_url'] . ', for user ' . $this->_config['ftp_login'], [], __METHOD__ . ':' . __LINE__);
            return true;
        } catch (Exception $e) {
            $this->_fileLogger->debug($e->getMessage(), [], __METHOD__ . ':' . __LINE__);
        }

        return false;
    }

    /**
     * Move to remote dir
     *
     * @param $data
     * @return bool
     */
    public function _moveToDir($data)
    {
        $dir = $this->getSystemConfig()->getRemoteRootDir();

        if ($dir) {
            $dir .= '/';
        }

        // Customers - Subscribers - Orders
        $dir .= ucfirst($data['params']->config->normalized_type) . 's';

        //If Dir exists on remote server
        if ($this->_client->cd($dir)) {
            $this->_fileLogger->debug('Moving to directory ' . $dir, [], __METHOD__ . ':' . __LINE__);
            return true;
        }
        //If Dir doesn't exists on remote server
        else {
            $this->_fileLogger->debug('Creating ' . $dir, [], __METHOD__ . ':' . __LINE__);
            //Create dir on remote server
            if ($this->_client->mkdir($dir)) {
                $this->_fileLogger->debug('Directory "' . $dir . '" created successfully', [], __METHOD__ . ':' . __LINE__);
                //Move to dir on remote server
                $this->_client->cd($dir);
                $this->_fileLogger->debug('Moved to directory ' . $dir, [], __METHOD__ . ':' . __LINE__);
                return true;
            } else {
                $this->_fileLogger->debug('Failed creating directory "' . $dir . '"', [], __METHOD__ . ':' . __LINE__);
                return false;
            }
        }
    }

    /**
     * Deconstructor
     */
    public function __deconstruct()
    {
        $this->_fileLogger->debug('Destroing SFTP Connection ', [], __METHOD__ . ':' . __LINE__);
        $this->_client->close();
    }
}
